"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

exports.__esModule = true;
exports.default = void 0;

var _regenerator = _interopRequireDefault(require("@babel/runtime/regenerator"));

var _asyncToGenerator2 = _interopRequireDefault(require("@babel/runtime/helpers/asyncToGenerator"));

var _extends2 = _interopRequireDefault(require("@babel/runtime/helpers/extends"));

var _createClass2 = _interopRequireDefault(require("@babel/runtime/helpers/createClass"));

/* eslint-disable class-methods-use-this */
var GoPatoDynamoDBLocalPlugin =
/*#__PURE__*/
function () {
  function GoPatoDynamoDBLocalPlugin(serverless, options) {
    this.serverless = serverless;
    this.options = options;
    this.commands = {
      'dynamodb-local': {
        commands: {
          install: {
            lifecycleEvents: ['installHandler']
          },
          start: {
            lifecycleEvents: ['setOfflineEnvironmentHandler', 'startHandler', 'migrateHandler']
          },
          stop: {
            lifecycleEvents: ['stopHandler']
          }
        }
      }
    };
    this.hooks = {
      'dynamodb-local:install:installHandler': this.startHandler.bind(this),
      'after:dynamodb-local:install:installHandler': this.stopHandler.bind(this),
      'dynamodb-local:start:setOfflineEnvironmentHandler': this.setOfflineEnvironmentHandler.bind(this),
      'dynamodb-local:start:startHandler': this.startHandler.bind(this),
      'dynamodb-local:start:migrateHandler': this.migrateHandler.bind(this),
      'dynamodb-local:stop:stopHandler': this.stopHandler.bind(this),
      'before:invoke:local:invoke': this.setOfflineEnvironmentHandler.bind(this)
    };
  }

  var _proto = GoPatoDynamoDBLocalPlugin.prototype;

  _proto.setOfflineEnvironmentHandler = function setOfflineEnvironmentHandler() {
    Object.assign(process.env, (0, _extends2.default)({}, this.serverless.service.provider.environment, {
      IS_OFFLINE: 'true'
    }));
  };

  _proto.validateDynamoDBConfig = function validateDynamoDBConfig() {
    if (!this.serverless.service.custom.dynamodb) {
      throw new Error('dynamodb config not found on serverless.custom');
    }

    if (!this.serverless.service.resources) {
      throw new Error('serverless.resources not found');
    }

    if (!this.serverless.service.resources.Resources) {
      throw new Error('serverless.resources.Resources not found');
    }

    if (!Object.values(this.serverless.service.resources.Resources).filter(function (resource) {
      return resource.Type === 'AWS::DynamoDB::Table';
    }).length) {
      throw new Error('could not find any AWS::DynamoDB::Table on serverless.resources.Resources');
    }
  };

  _proto.startHandler =
  /*#__PURE__*/
  function () {
    var _startHandler = (0, _asyncToGenerator2.default)(
    /*#__PURE__*/
    _regenerator.default.mark(function _callee() {
      return _regenerator.default.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              this.validateDynamoDBConfig();
              _context.next = 3;
              return this.utils.registerTables(this.dynamodbTables);

            case 3:
              _context.next = 5;
              return this.utils.start();

            case 5:
            case "end":
              return _context.stop();
          }
        }
      }, _callee, this);
    }));

    return function startHandler() {
      return _startHandler.apply(this, arguments);
    };
  }();

  _proto.migrateHandler =
  /*#__PURE__*/
  function () {
    var _migrateHandler = (0, _asyncToGenerator2.default)(
    /*#__PURE__*/
    _regenerator.default.mark(function _callee2() {
      var _this = this;

      return _regenerator.default.wrap(function _callee2$(_context2) {
        while (1) {
          switch (_context2.prev = _context2.next) {
            case 0:
              if (this.options.migrate) {
                _context2.next = 2;
                break;
              }

              return _context2.abrupt("return", false);

            case 2:
              return _context2.abrupt("return", Promise.all(this.dynamodbTables.map(function (table) {
                return _this.utils.setupTable(table.config.TableName, true);
              })));

            case 3:
            case "end":
              return _context2.stop();
          }
        }
      }, _callee2, this);
    }));

    return function migrateHandler() {
      return _migrateHandler.apply(this, arguments);
    };
  }();

  _proto.stopHandler = function stopHandler() {
    return this.utils.stop();
  };

  (0, _createClass2.default)(GoPatoDynamoDBLocalPlugin, [{
    key: "utils",
    get: function get() {
      /* eslint-disable global-require */
      return require('@gopato/serverless-dynamodb-local-utils');
    }
  }, {
    key: "dynamodbTables",
    get: function get() {
      var _this2 = this;

      return Object.values(this.serverless.service.resources.Resources).filter(function (_ref) {
        var Type = _ref.Type;
        return Type === 'AWS::DynamoDB::Table';
      }).map(function (_ref2) {
        var config = _ref2.Properties;
        return {
          config: config,
          seeds: _this2.serverless.service.custom.dynamodb.seeds.filter(function (s) {
            return s.table === config.TableName;
          }).map(function (s) {
            return s.sources;
          }).reduce(function (acc, curr) {
            return acc.concat(curr);
          }, [])
        };
      });
    }
  }]);
  return GoPatoDynamoDBLocalPlugin;
}();

exports.default = GoPatoDynamoDBLocalPlugin;
module.exports = exports.default;